'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _drop = require('lodash/drop');

var _drop2 = _interopRequireDefault(_drop);

var _dropRight = require('lodash/dropRight');

var _dropRight2 = _interopRequireDefault(_dropRight);

var _first = require('lodash/first');

var _first2 = _interopRequireDefault(_first);

var _last = require('lodash/last');

var _last2 = _interopRequireDefault(_last);

var _forEach = require('lodash/forEach');

var _forEach2 = _interopRequireDefault(_forEach);

var _isFunction = require('lodash/isFunction');

var _isFunction2 = _interopRequireDefault(_isFunction);

var _cursorHelpers = require('./cursor-helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

exports.default = () => {
  return function (query = {}, args = {}, opts = {}) {
    var _this = this;

    return new Promise((() => {
      var _ref = _asyncToGenerator(function* (resolve, reject) {
        if (args.first !== undefined && args.last !== undefined) {
          return reject(new TypeError('first and last cannot be set at the same time.'));
        }

        let { after, before } = args;

        try {
          if (after) {
            after = yield (0, _cursorHelpers.validateCursor)(after);
          }
          if (before) {
            before = yield (0, _cursorHelpers.validateCursor)(before);
          }
        } catch (err) {
          return reject(err);
        }

        query = { $and: [query] };

        const anyCursor = after || before;
        if (args.filters || anyCursor && anyCursor.filters) {
          const filtersToUse = anyCursor && anyCursor.filters ? anyCursor.filters : args.filters;
          (0, _forEach2.default)(filtersToUse, function (value, name) {
            let condition = { [name]: value };
            if (!opts.filterFields || !opts.filterFields[name]) {
              return;
            }

            if ((0, _isFunction2.default)(opts.filterFields[name])) {
              condition = opts.filterFields[name](value);
            }

            query.$and.push(condition);
          });
        }

        if (args.search) {
          const s = (opts.searchFields || []).map(function (v) {
            const filter = {};
            filter[v] = { '$regex': `${args.search}`, '$options': 'i' };
            return filter;
          });

          query.$and.push({ $or: s });
        }

        const totalCount = yield _this.countDocuments(query);
        const { first, last } = args;
        const limit = first || last || 1000;
        const isDescSort = args.sort && args.sort.order === 'desc';
        var hasNextPage = false,
            hasPreviousPage = false;
        var skip = 0;
        var sort = {};

        const buildCursorCondition = function (cursor, way) {
          if (cursor.sort) {
            let { _id, sort: { field, value } } = cursor;
            return {
              $or: [{
                [field]: { [way]: value }
              }, {
                [field]: value,
                _id: { [way]: _id }
              }]
            };
          }

          return { _id: { [way]: cursor._id } };
        };

        var dataQuery = query;
        if (after) {
          dataQuery.$and.push(buildCursorCondition(after, isDescSort ? '$lt' : '$gt'));
          hasPreviousPage = true;
        }

        if (before) {
          dataQuery.$and.push(buildCursorCondition(before, isDescSort ? '$gt' : '$lt'));
          hasNextPage = true;
        }

        const hasOffset = !(hasNextPage && hasPreviousPage);

        if (last) {
          let offset = totalCount - last - (hasOffset ? 1 : 0);
          if (before) {
            const dataCount = yield _this.countDocuments(dataQuery);
            offset = dataCount - last - 1;
          }
          skip = offset < 0 ? 0 : offset;
        }

        if (args.sort) {
          let { order } = args.sort;
          sort = {
            [args.sort.by]: order,
            _id: order
          };
        }

        let data = yield _this.find(dataQuery).sort(sort).skip(skip).limit(limit + (hasOffset ? 1 : 0));

        if (hasOffset && data.length > limit) {
          if (first) {
            hasNextPage = true;
          }

          if (last) {
            hasPreviousPage = true;
          }

          if (first || after) {
            data = (0, _dropRight2.default)(data);
          } else if (last || before) {
            data = (0, _drop2.default)(data);
          }
        }

        const edges = data.map(function (v) {
          return {
            cursor: (0, _cursorHelpers.generateCursor)(v, args.sort, args.filters),
            node: v
          };
        });

        resolve({
          totalCount,
          pageInfo: {
            hasNextPage,
            hasPreviousPage,
            startCursor: (0, _first2.default)(edges) ? (0, _first2.default)(edges).cursor : null,
            endCursor: (0, _last2.default)(edges) ? (0, _last2.default)(edges).cursor : null
          },
          edges
        });
      });

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    })());
  };
};